import { Tr } from "pi_ucore/db/mgr";
import { read } from "pi_common_tools/util/db_tool";
import { getCurrentRid } from "pi_common_tools/main";
import { getChooseCfg, getRoundCfg } from "../base/cfg_tool";
import { BigChooseRecord } from "game_mj_lottery/server/lottery.struct";

/**
 * 获取玩家活动表主键
 * @param rid 玩家rid
 * @param key 活动key
 * @returns 
 */
export const getPrimaryKey = (key: string, rid?: number) => {
    rid = rid || getCurrentRid();
    return key + '_' + rid;
}

/**
 * 根据轮次id获取轮次配置信息
 * @param round 轮次id
 * @returns 
 */
export const roundCfg = (round: number) => {
    const cfg = getRoundCfg(round);
    if (cfg) return cfg;
    const id = Array.from(getRoundCfg().keys()).sort((v1, v2) => v2 - v1)[0];
    return getRoundCfg(id);
}

/**
 * 当前轮次大奖
 * @param tr 
 * @param uid 
 * @param key 
 * @param roundId 
 * @returns 
 */
export const curRoundBigAward = async (tr: Tr, uid: number, key: string, roundId: number) => {
    const cfg = roundCfg(roundId);
    if (cfg.BigAward && cfg.BigAward.length > 0) return cfg.BigAward;
    const primaryKey = getPrimaryKey(key, uid);
    const chooseRecord = await read(tr, BigChooseRecord, primaryKey);
    const chooseCfg = getChooseCfg(chooseRecord.curBigId);
    if (!chooseCfg) return null;
    return chooseCfg.ChooseBigAward;
}