import { Tr } from "pi_ucore/db/mgr";
import { DateMgr } from "pi_business_tool/date_mgr";
import { getCurrentRid } from "pi_common_tools/main";
import { getPrimaryKey, roundCfg, curRoundBigAward } from "../base/base";

import { filter } from "../cfg_config";
import { writes, read } from "pi_common_tools/util/db_tool";
import { init as lInit, go_next_round, lottery } from "./base";
import { getLuckDrawCfg, getBaseCfg, getProbabilityCfg, getChooseCfg, getRoundCfg } from "../base/cfg_tool"
import { Lottery, drawArg, arg as pArg, BigChooseRecord } from "game_mj_lottery/server/lottery.struct";

// #[rpc=rpcServer,src=*,rpcWrapper=read,rpcMode=check]
export const mainPage = async (tr: Tr, rid: number, arg: pArg) => {
    const primaryKey = getPrimaryKey(arg.key, rid);
    const lottery = await read(tr, Lottery, primaryKey);
    const baseCfg = getBaseCfg();
    const cfg = roundCfg(lottery.curRound);
    const luckCfg = getLuckDrawCfg(baseCfg.luck_draw_id);
    const now = DateMgr.now();
    const coolTime = luckCfg.cooling ? DateMgr.dailyZero(now) + (arg.refHour + luckCfg.cooling) * DateMgr.OneHourMs : 0;
    let free_count = lottery.freeCount;
    if (now >= lottery.coolTime && luckCfg.cooling) {
        const coolTime = DateMgr.dailyZero(now) + (luckCfg.cooling + arg.refHour) * DateMgr.OneHourMs;  //下一天刷新时间
        const coolTimes = Math.floor((coolTime - lottery.coolTime) / DateMgr.OneDayMs);               //免费次数天数
        free_count += (luckCfg.freeCount * coolTimes);
    }
    const big_award = await curRoundBigAward(tr, rid, arg.key, lottery.curRound);

    const isSel = !cfg.BigAward || cfg.BigAward.length <= 0;

    return {
        now,
        isSel,
        big_award,
        free_count,
        area: cfg.area,
        refTime: coolTime,
        cost: luckCfg.cost,
        record: lottery.record,
        round: lottery.curRound,
        tenCost: luckCfg.tenCost,
        lottery: lottery.record.filter(v => v === 0).length
    }
}

// #[rpc=rpcServer,src=*,rpcWrapper=write,rpcMode=check]
export const show = async (tr: Tr, rid: number, key: string) => {
    const primaryKey = getPrimaryKey(key, rid);
    const lottery = await read(tr, Lottery, primaryKey);
    const baseCfg = getBaseCfg();
    const cfg = roundCfg(lottery.curRound);
    let big_award = [], isSel = true;
    if (cfg.BigAward && cfg.BigAward.length > 0) {
        isSel = false;
        big_award = big_award.concat(cfg.BigAward);
    } else {
        const chooseCfg = getChooseCfg();
        for (let v of chooseCfg) {
            const is = await filter(v[1].id);
            if (!is) continue;
            big_award = big_award.concat(v[1].ChooseBigAward);
        }
    }
    const list = [];
    const cfgs = getProbabilityCfg();
    for (let v of cfgs) {
        if (v[1].kc_id.includes(baseCfg.luck_draw_id)) {
            list.push(v[1].award)
        }
    }
    return {
        list,
        isSel,
        big_award
    }
}

// #[rpc=rpcServer,src=*,rpcWrapper=write,rpcMode=check]
export const showAward = async (tr: Tr, rid: number, key: string) => {
    const primaryKey = getPrimaryKey(key, rid);
    const lottery = await read(tr, Lottery, primaryKey);
    const chooseCfg = getChooseCfg();
    const chooseAwrds = [];
    for (let v of chooseCfg) {
        const is = await filter(v[1].id);
        if (!is) continue;
        chooseAwrds.push(...v[1].ChooseBigAward);
    }
    const roundCfg = getRoundCfg();
    const list = [];
    let maxId = 0;
    for (let v of roundCfg) {
        const notSel = v[1].BigAward && Array.isArray(v[1].BigAward) && v[1].BigAward.length > 0;
        list.push({
            title: `第${v[1].id}轮${!notSel ? '及以后' : ''}`,
            isGet: !notSel ? false : lottery.curRound > v[1].id,
            isSel: !notSel,
            award: notSel ? v[1].BigAward : chooseAwrds
        })
        if (!notSel) {
            maxId = v[1].id;
            break;
        }
    }
    return {
        list,
        curRound: Math.min(lottery.curRound, maxId)
    };
}

// #[rpc=rpcServer,src=*,rpcWrapper=write,rpcMode=check]
export const select = async (tr: Tr, rid: number, key: string) => {
    const primaryKey = getPrimaryKey(key, rid);
    const chooseRecord = await read(tr, BigChooseRecord, primaryKey);
    let selAward, selId = chooseRecord.curBigId;
    const list = [];
    const cfgs = getChooseCfg();
    for (let v of cfgs) {
        const curTime = chooseRecord.record.get(v[1].id) || 0;
        if (chooseRecord.curBigId === v[1].id) {
            selAward = v[1].ChooseBigAward
        }
        const is = await filter(v[1].id);
        if (!is) continue;
        list.push({
            curTime,
            id: v[1].id,
            maxTime: v[1].ChooseTimes,
            award: v[1].ChooseBigAward
        })
    }
    list.sort((v1, v2) => {
        const has1 = !v1.maxTime ? 2 : (v1.maxTime - v1.curTime) > 0 ? 1 : 0;
        const has2 = !v2.maxTime ? 2 : (v2.maxTime - v2.curTime) > 0 ? 1 : 0;
        return has2 - has1;
    })

    return {
        list,
        selId,
        selAward
    }
}

// #[rpc=rpcServer,src=*,rpcWrapper=read,rpcMode=remote]
export const initLottery = async (tr: Tr, rid: number, arg: pArg) => {
    const items = [];
    await lInit(tr, rid, arg.key, items, 0, arg.refHour);
    if (items && items.length > 0) await writes(tr, items);
}

// #[rpc=rpcServer,src=*,rpcWrapper=write,rpcMode=fork]
export const goNextRound = async (tr: Tr, rid: number, key: string) => {
    const items = [];
    const r = await go_next_round(tr, rid, key, items, 0);
    if (!r) return false;
    if (items && items.length > 0) await writes(tr, items);
    return true;
}

// #[rpc=rpcServer,src=*,rpcWrapper=write,rpcMode=remote]
export const draw = async (tr: Tr, rid: number, arg: drawArg) => {
    const items = [];
    const r = await lottery(tr, rid, arg.key, items, arg.times, arg.index, arg.refHour)
    if (!r) return false;
    if (items && items.length > 0) await writes(tr, items);
    return r;
}