import { Tr } from "pi_ucore/db/mgr";
import { Struct } from "pi_serialization/struct_mgr";
import { IStruct, read } from "pi_common_tools/util/db_tool";

import { DropDb } from "../cfg_config";
import { getDropCfg } from "../base/cfg_tool";

const getMultipleProp = (awardCfg, awardList, multiple: number) => {
    if (Array.isArray(awardCfg)) {
        if (Array.isArray(awardCfg[0])) {//奖励是二维数组
            awardCfg.forEach(element => {
                let prop = element.slice(0);
                prop[prop.length - 1] = Math.floor(prop[prop.length - 1] * multiple);
                awardList.push(prop);
            })
        } else {//奖励是一维数组
            let prop = awardCfg.slice(0);
            prop[prop.length - 1] = Math.floor(prop[prop.length - 1] * multiple);
            awardList.push(prop);
        }
    }
}

const calcCount = (val): number => {
    if (!val.min_rate) { return 1 };
    if (randomInt(1, 10000) < val.min_rate) {
        let count = randomInt(val.min_min_value, val.min_max_value);
        return count;
    } else {
        let count = randomInt(val.max_min_value, val.max_max_value);
        return count;
    }
}

const dropShare = (cfgList) => {
    if (cfgList.length === 0) { return { award: {}, count: 0, index: 0 } };
    let weights_val = 0, groupObj = [];//权重总值
    cfgList.forEach((val, index) => {
        if (val.prob !== 0) {
            groupObj.push({ propcfg: val, site: index, min: weights_val, max: weights_val + val.prob });
            weights_val += val.prob;
        }
    });
    let random_re = randomInt(1, weights_val);
    for (let v of groupObj) {
        if (random_re > v.min && random_re <= v.max) {
            return { award: v.propcfg.award, count: calcCount(v.propcfg), index: v.site }
        };
    };
}

//权重重复多次掉落模式
const repeatShareDrop = (dropInfo, count) => {
    if (dropInfo.length === 0) { return {} }
    let totalAward = [];
    while (count !== 0) {
        let ramdomObj = dropShare(dropInfo);
        getMultipleProp(ramdomObj.award, totalAward, ramdomObj.count);
        count -= 1
    }
    return totalAward;
}

//获得配置奖励消耗  不涉及需要额外参数的奖励
const getProp = (awardCfg, awardList) => {
    if (Array.isArray(awardCfg)) {
        if (Array.isArray(awardCfg[0])) {//奖励是二维数组
            awardCfg.forEach(val => {
                if (val.length !== 0) { awardList.push(val) };
            });
        } else {//奖励是一维数组
            if (awardCfg.length !== 0) { awardList.push(awardCfg) };
        }
    }
}

//权重唯一多次掉落模式
const uniqueShareDrop = (dropInfo, count) => {
    if (dropInfo.length === 0) { return {} };
    let totalAward = [];
    while (count !== 0) {
        let ramdomObj = dropShare(dropInfo);
        getMultipleProp(ramdomObj.award, totalAward, ramdomObj.count);
        dropInfo.splice(ramdomObj.index, 1);
        count -= 1
    }
    return totalAward;
}

//真随机
const dropRandom = (val) => {
    let dropValue = val.prob;
    if (randomInt(1, 10000) < dropValue) {//随机到了
        //开始随机道具出现几份
        let count = calcCount(val);
        return count
    } else { return 0 }
}

export const randomInt = (min: number, max: number) => {
    return Math.floor(Math.random() * (max - min + 1)) + min
}

export const dropTwo = async (tr: Tr, groupList: number[], addConfig: object = {}) => {
    let dropAward = [];
    if (groupList.length === 0) { return dropAward };
    for (let groupID of groupList) {
        const dropDb = await read(tr, DropDb, groupID);
        //读取内存表中的信息
        if (!dropDb) { return dropAward };
        let dropInfo = dropDb.configList || [], dropMode = dropDb.dropMode, mode_param = dropDb.modeArg;
        if (JSON.stringify(addConfig) !== "{}") { dropInfo.push(addConfig) };
        if (dropInfo.length === 0) continue;
        if (dropMode === "share" && mode_param === 0) {//share掉落模式  权重随机
            let ramdomObj = dropShare(dropInfo);
            getMultipleProp(ramdomObj.award, dropAward, ramdomObj.count);
        } else if (dropMode === "share" && mode_param !== 0) {//share_count掉落模式  权重重复多次
            let randomA = repeatShareDrop(dropInfo, mode_param);
            getProp(randomA, dropAward);
        } else if (dropMode === "share_unique" && mode_param !== 0) {//share_unique_count掉落模式
            let dropL = uniqueShareDrop(dropInfo.slice(0), mode_param);
            getProp(dropL, dropAward);
        } else if (dropMode === "random") {//random掉落模式    真随机
            dropInfo.forEach(val => {
                let count = dropRandom(val);
                count !== 0 && getMultipleProp(val.award, dropAward, count);
            });
        }
    }
    return dropAward;
}

// 初始抽卡内存表
export const initDropDb = (items: [IStruct<Struct>, any, any][]) => {
    let cacheMap: Map<number, object> = new Map();
    let allConf = getDropCfg();
    for (let elData of allConf) {
        let cfgInfo = elData[1];
        let groupID = cfgInfo.drop_id, mode = cfgInfo.drop_mode, modeParam = cfgInfo.drop_mode_param;
        let infoObj: object;
        if (!cacheMap.has(groupID)) {
            infoObj = { mode: "", param: 0, list: [] };
            cacheMap.set(groupID, infoObj);
        } else {
            infoObj = cacheMap.get(groupID)
        };
        if (mode !== "" && modeParam !== 0 && infoObj["mode"] === "" && infoObj["param"] === 0) {//取该随机组中第一次出现的掉落模式和掉落参数
            infoObj["mode"] = mode, infoObj["param"] = modeParam;
        };
        infoObj["list"].push(cfgInfo);
    }
    //将缓存Map中的信息组装成表
    for (let ele of cacheMap) {
        let groupID = ele[0], Info = ele[1];
        let tableInfo = new DropDb();
        tableInfo.key = groupID;
        tableInfo.dropMode = Info["mode"];
        tableInfo.modeArg = Info["param"];
        tableInfo.configList = Info["list"];
        items.push([DropDb, groupID, tableInfo]);
    };
}